import json

from sqlalchemy.ext.associationproxy import association_proxy
from sqlalchemy.schema import UniqueConstraint

from app import db
from .role import Role
from .notification_player import NotificationPlayer
from .player_found_objective import PlayerFoundObjective
from .review import Review

class GamePlayer(db.Model):
    __tablename__ = 'game_player'
    id = db.Column(db.Integer, primary_key=True)
    game_id = db.Column(db.Integer, db.ForeignKey('game.id'), nullable=False)
    user_id = db.Column(db.Integer, db.ForeignKey('user.id'), nullable=False)
    role = db.Column(db.Enum(Role), server_default=Role(0).name, nullable=False)
    game = db.relationship('Game', back_populates='players')
    user = db.relationship('User', back_populates='user_games')
    __table_args__ = (UniqueConstraint('game_id', 'user_id', name='_game_user_uc'),
                     )

    player_notifications = db.relationship(
        'NotificationPlayer',
        back_populates='recipient',
        cascade='save-update, merge, delete, delete-orphan')
    notifications = association_proxy('player_notifications', 'notification',
        creator=lambda notification: NotificationPlayer(notification=notification))

    player_found_objectives = db.relationship(
        'PlayerFoundObjective',
        back_populates='game_player',
        cascade='save-update, merge, delete, delete-orphan')
    found_objectives = association_proxy('player_found_objectives', 'objective',
        creator=lambda objective: PlayerFoundObjective(objective=objective))

    caught_by_players = association_proxy('player_caught_by_players', 'catching_player')
    caught_players = association_proxy('player_caught_players', 'caught_player')

    def last_location(self, offset=None):
        '''
        Returns game_player's last recorded location within game start- and end time.

            Parameters:
                offset (int): Offset in minutes. Only locations older than this amount of minutes will be returned.
        '''
        # pylint: disable=no-member
        return self.user.last_location(self.game, offset)

    def locations_during_game(self):
        # pylint: disable=no-member
        return self.user.locations_during_game(self.game)

    def encode_objectives(self):
        # pylint: disable=no-member
        objectives = ['[']
        for objective in self.game.objectives:
            obj = {
                'name' : objective.name,
                'longitude' : objective.longitude,
                'latitude' : objective.latitude,
                'found' : objective in self.found_objectives}
            objectives.append(json.dumps(obj))
            objectives.append(',')
        return ''.join(objectives)[:-1] + ']'

    def accepted_caught_players(self):
        return [pcp.caught_player
                for pcp in self.player_caught_players
                if pcp.review == Review.accepted]

    def accepted_caught_by_players(self):
        return [pcp.catching_player
                for pcp in self.player_caught_by_players
                if pcp.review == Review.accepted]
